@@ -7,7 +7,7 @@ use strict;
 use Alzabo::Exceptions;
 use Time::HiRes qw( time );
 
-$VERSION = sprintf '%2d.%02d', q$Revision: 1.15 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf '%2d.%02d', q$Revision: 1.16 $ =~ /(\d+)\.(\d+)/;
 
 1;
 
@@ -23,7 +23,6 @@ sub new
     $SELF = bless {}, $class;
     $SELF->_init(@_);
     $SELF->{obj_times} = {};
-    $SELF->{id_times} = {};
 
     return $SELF;
 }
@@ -37,34 +36,34 @@ sub register_store
 {
     my $self = shift;
     my $obj = shift;
-    my $id = $obj->id;
     my $time = shift;
 
+    my $id = $obj->id;
+    my $cache_id = $obj->cache_id;
+
     $time = sprintf('%11.23f', defined $time ? $time : time);
 
-    $self->{obj_times}{$obj} = $time;
-
-    return if
-	exists $self->{id_times}{$id};
+    $self->{obj_times}{$cache_id} = $time;
 
     # don't overwrite
     $self->update( $id => $time, 0 );
-    $self->{id_times}{$id} = $time;
 }
 
 sub is_expired
 {
     my $self = shift;
     my $obj = shift;
+
     my $id = $obj->id;
+    my $cache_id = $obj->cache_id;
 
     my $sync_time = $self->sync_time($id);
 
-    return 1 if exists $self->{obj_times}{$obj} && $self->{obj_times}{$obj} < $sync_time;
+    return 0 if exists $self->{obj_times}{$cache_id} && $self->{obj_times}{$cache_id} >= $sync_time;
 
-    return 1 if exists $self->{id_times}{$id} && $self->{id_times}{$id} < $sync_time;
+    return 1 if exists $self->{obj_times}{$cache_id} && $self->{obj_times}{$cache_id} < $sync_time;
 
-    return 1 if $sync_time && ! ( exists $self->{id_times}{$id} || exists $self->{obj_times}{$obj} );
+    return 1 if $sync_time && ! exists $self->{obj_times}{$cache_id};
 
     return 0;
 }
@@ -74,25 +73,26 @@ sub register_refresh
     my $self = shift;
     my $obj = shift;
     my $id = $obj->id;
+    my $cache_id = $obj->cache_id;
 
-    return unless exists $self->{obj_times}{$obj};
+    return unless exists $self->{obj_times}{$cache_id};
 
     my $time = sprintf( '%11.23f', time );
-    $self->{obj_times}{$obj} = $time;
+    $self->{obj_times}{$cache_id} = $time;
 }
 
 sub register_change
 {
     my $self = shift;
     my $obj = shift;
-    my $id = $obj->id;
     my $time = shift;
 
-    return unless exists $self->{id_times}{$id};
+    my $id = $obj->id;
+    my $cache_id = $obj->cache_id;
 
     $time = sprintf('%11.23f', defined $time ? $time : time);
 
-    $self->{id_times}{$id} = $self->{obj_times}{$obj} = $time;
+    $self->{obj_times}{$cache_id} = $time;
     $self->update( $id => $time, 1 );
 }
 
@@ -101,12 +101,10 @@ sub register_delete
     my $self = shift;
     my $obj = shift;
     my $id = $obj->id;
-
-    return unless exists $self->{id_times}{$id};
+    my $cache_id = $obj->cache_id;
 
     $self->update( $id => -1, 1 );
-    delete $self->{id_times}{$id};
-    delete $self->{obj_times}{$obj};
+    delete $self->{obj_times}{$cache_id};
 }
 
 sub is_deleted
@@ -123,15 +121,15 @@ sub delete_from_cache
 {
     my $self = shift;
     my $obj = shift;
+    my $cache_id = $obj->cache_id;
 
-    delete $self->{obj_times}{$obj};
+    delete $self->{obj_times}{$cache_id};
 }
 
 sub clear
 {
     my $self = shift;
 
-    $self->{id_times} = {};
     $self->{obj_times} = {};
 }
 
